﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using Canonical.UbuntuOne.Common.Validation;
using NUnit.Framework;

namespace Canonical.UbuntuOne.Common.Tests.Validation
{
    [TestFixture]
    public class EqualityExtensionsValidation
    {
        #region Helper class

        private interface ICurrency : IComparable
        {
            int Amount { get; set; }
        }

        private class Currency : ICurrency 
        {
            public int Amount { get; set; }
            public int CompareTo(object obj)
            {
                var other = obj as Currency;
                if (other != null)
                    return Amount.CompareTo(other.Amount);
                throw new ArgumentException();
            }
        }
        #endregion

        #region Tests

        [Test]
        public void IsGreaterThanTrueTest()
        {
            ValidateArgs.Begin()
                .IsGreaterThan(4, 3, "should be greater")
                .Check();
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void IsGreaterThanFalseTest()
        {
            ValidateArgs.Begin()
                .IsGreaterThan(3, 4, "should be greater")
                .Check();
        }

        [Test]
        public void IsGreaterOrEqualTrueTest()
        {
            ValidateArgs.Begin()
                .IsGreaterOrEqualThan(4, 4, "Is equal")
                .IsGreaterOrEqualThan(5,4, "Is Greater")
                .Check();
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void IsGreaterOrEqualFalseTest()
        {
            ValidateArgs.Begin()
                .IsGreaterOrEqualThan(3, 4, "Is smaller")
                .Check();
                
        }

        [Test]
        public void IsEqualTrueTest()
        {
            ValidateArgs.Begin()
                .IsEqual(3, 3)
                .Check();
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void IsEqualFalseTest()
        {
            ValidateArgs.Begin()
                .IsEqual(3, 4)
                .Check();
        }
        
        [Test]
        public void IsEqualCollectionTrueTest()
        {
            var currency = new Currency {Amount = 4};
            var collection = new List<Currency>
            {
                new Currency {Amount = 4},
                new Currency {Amount = 3},
                new Currency {Amount = 2},
                new Currency {Amount = 1}
            };
            ValidateArgs.Begin()
                .IsEqual(currency, collection)
                .Check();
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void IsEqualCollectionFalseTest()
        {
            var currency = new Currency { Amount = 8 };
            var collection = new List<Currency>
            {
                new Currency {Amount = 4},
                new Currency {Amount = 3},
                new Currency {Amount = 2},
                new Currency {Amount = 1}
            };
            ValidateArgs.Begin()
                .IsEqual(currency, collection)
                .Check();
        }

        [TestCase(2)]
        [TestCase(25)]
        public void IsEqualIComparableTrueTest(int amount)
        {
            var firstCurrency = new Currency {Amount = amount};
            var secondCurrency = new Currency { Amount = amount };
            ValidateArgs.Begin()
                .IsEqual<ICurrency>(firstCurrency, secondCurrency)
                .Check();
        }

        [TestCase(3, 5)]
        [TestCase(34, 77)]
        [ExpectedException(typeof(ValidationException))]
        public void IsEqualIComparableFalseTest(int firstAmount, int secondAmount)
        {
            var firstCurrency = new Currency { Amount = firstAmount };
            var secondCurrency = new Currency { Amount = secondAmount };
            ValidateArgs.Begin()
                .IsEqual<ICurrency>(firstCurrency, secondCurrency)
                .Check();
        }

        [TestCase(2)]
        [TestCase(25)]
        [ExpectedException(typeof(ValidationException))]
        public void IsNotEqualFalseTest(int amount)
        {
            var firstCurrency = new Currency { Amount = amount };
            var secondCurrency = new Currency { Amount = amount };
            ValidateArgs.Begin()
                .IsNotEqual<ICurrency>(firstCurrency, secondCurrency)
                .Check();
        }

        [TestCase(3, 5)]
        [TestCase(34, 77)]
        public void IsNotEqualTrueTest(int firstAmount, int secondAmount)
        {
            var firstCurrency = new Currency { Amount = firstAmount };
            var secondCurrency = new Currency { Amount = secondAmount };
            ValidateArgs.Begin()
                .IsNotEqual<ICurrency>(firstCurrency, secondCurrency)
                .Check();
        }

        [Test]
        public void IsNotEqualCollectionTrueTest()
        {
            var currency = new Currency { Amount = 8 };
            var collection = new List<Currency>
            {
                new Currency {Amount = 4},
                new Currency {Amount = 3},
                new Currency {Amount = 2},
                new Currency {Amount = 1}
            };
            ValidateArgs.Begin()
                .IsNotEqual(currency, collection)
                .Check();
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void IsNotEqualCollectionFalseTest()
        {
            var currency = new Currency { Amount = 4 };
            var collection = new List<Currency>
            {
                new Currency {Amount = 4},
                new Currency {Amount = 3},
                new Currency {Amount = 2},
                new Currency {Amount = 1}
            };
            ValidateArgs.Begin()
                .IsNotEqual(currency, collection)
                .Check();   
        }

        [TestCase("Manuel", "M.*")]
        [TestCase("manuel@canonical.com", ".*@canonical.com")]
        public void MatchesTrueTest(string input, string pattern)
        {
            var regexp = new Regex(pattern);
            ValidateArgs.Begin()
                .Matches(input, regexp)
                .Check();
        }

        [TestCase("Luis", "M.*")]
        [TestCase("manuel@gmail.com", ".*@canonical.com")]
        [ExpectedException(typeof(ValidationException))]
        public void MatchesFalseTest(string input, string pattern)
        {
            var regexp = new Regex(pattern);
            ValidateArgs.Begin()
                .Matches(input, regexp)
                .Check();
        }

        #endregion

    }
}
