﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using log4net;

namespace Canonical.UbuntuOne.Common.Container
{
    /// <summary>
    /// Static gateway to the IoC container
    /// </summary>
    public static class ObjectsContainer
    {
        private static readonly ILog _logger = LogManager.GetLogger(typeof(ObjectsContainer));
        private static IContainer _container;

        /// <summary>
        /// Initializes the specified container.
        /// </summary>
        /// <param name="container">The container.</param>
        public static void Initialize(IContainer container)
        {
            _container = container;
        }

        /// <summary>
        /// Remove any object in the container
        /// You must reinitialize when continue to use the container
        /// </summary>
        public static void Clear()
        {
            if (_container != null)
            {
                _container.Clear();
                _container = null;
            }
        }

        /// <summary>
        /// Gets the mapper.
        /// </summary>
        /// <typeparam name="From">Type to convert from</typeparam>
        /// <typeparam name="To">Type to convert to</typeparam>
        /// <returns>A mapper which can convert a specified type</returns>
        public static IMapper<From, To> GetMapper<From, To>()
        {
            var result = GetImplementationOf<IMapper<From, To>>();
            return result;

        }

        /// <summary>
        /// Registers the specified implementation for a type.
        /// </summary>
        /// <typeparam name="T">Type to register an implementation for.</typeparam>
        /// <param name="implementationToRegister">The implementation to register.</param>
        public static void Register<T>(T implementationToRegister)
        {
            if (_container == null)
            {
                throw new InvalidOperationException(ContainerResources.NotInitializedContainer);
            }
            _container.Register(implementationToRegister);
        }

        /// <summary>
        /// Gets the implementation of a type.
        /// </summary>
        /// <typeparam name="T">Type to retrieve</typeparam>
        /// <returns>Implementation of a type</returns>
        public static T GetImplementationOf<T>() where T : class
        {
            if (_container == null)
            {
                throw new InvalidOperationException(ContainerResources.NotInitializedContainer);
            }

            try
            {
                var result = _container.GetImplementationOf<T>();
                return result;
            }
            catch (Exception e)
            {
                throw new UnsatisfiedDependencyException(string.Format(ContainerResources.NoElementsRegistered, typeof(T).Name), e);
            }
        }
    }
}
