﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Collections;
using System.Linq;
using Spring.Context;
using Spring.Context.Support;
using Spring.Objects.Factory.Config;
using Spring.Objects.Factory.Support;

namespace Canonical.UbuntuOne.Common.Container
{
    /// <summary>
    /// Spring implementation of IContainer
    /// On construction all Spring object configuration will be initialized
    /// </summary>
    public class SpringContainer : IContainer, IDisposable
    {

        #region Private variables

        private readonly IApplicationContext _applicationContext;
        private bool _disposed;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SpringContainer"/> class.
        /// Get the Spring config from the config file
        /// </summary>
        public SpringContainer()
            : this(ContextRegistry.GetContext())
        {
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SpringContainer"/> class.
        /// The spring config is provided by a list of context strings
        /// </summary>
        /// <example>
        /// <![CDATA[
        /// var container = new SpringContainer("config://spring/objects",
        ///                                     "assembly://Artilium.Arta.Core.Data/Artilium.Arta.Core.Data/CoreDataAccess.xml");
        /// ]]>
        /// </example>
        /// <param name="config">The configs.</param>
        public SpringContainer(string[] config)
        {
            var xmlCtx = new XmlApplicationContext(config);
            _applicationContext = new StaticApplicationContext(xmlCtx);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SpringContainer"/> class.
        /// The spring context is provided by a custom IApplicationContext
        /// </summary>
        /// <param name="context">The context.</param>
        public SpringContainer(IApplicationContext context)
        {
            _applicationContext = context;
        }

        #endregion

        /// <summary>
        /// Registers the specified implementation for a type.
        /// </summary>
        /// <typeparam name="T">Type to register an implementation for.</typeparam>
        /// <param name="implementationToRegister">The implementation to register.</param>
        public void Register<T>(T implementationToRegister)
        {
            var ctx = _applicationContext as AbstractApplicationContext;
            if (ctx == null)
            {
                throw new InvalidOperationException(ContainerResources.RegsitrationNotSupported);
            }
            ctx.ObjectFactory.RegisterSingleton(typeof(T).Name, implementationToRegister);
        }

        /// <summary>
        /// Registers the specified type and use autowiring for dependency injection
        /// </summary>
        /// <typeparam name="T"></typeparam>
        public void Register<T>()
        {
            var objectFactory = new DefaultObjectDefinitionFactory();
            var objDef = objectFactory.CreateObjectDefinition(typeof(T).FullName, null, AppDomain.CurrentDomain);
            objDef.AutowireMode = AutoWiringMode.AutoDetect;
            var ctx = _applicationContext as GenericApplicationContext;
            if (ctx != null)
            {
                // register the object definition base on the type name
                ctx.RegisterObjectDefinition(typeof(T).Name, objDef);
            }
        }

        /// <summary>
        /// Remove all objects in container. Normally only used in unit tests
        /// 
        /// </summary>
        public void Clear()
        {
            if (_applicationContext == null)
                return;
            ContextRegistry.Clear();
            _applicationContext.Dispose();
        }

        /// <summary>
        /// Gets the implementation of a type based on its name.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="name">The name.</param>
        /// <returns></returns>
        public T GetImplementationOf<T>(string name) where T : class
        {
            return _applicationContext.GetObject(name, typeof(T)) as T;
        }

        /// <summary>
        /// Gets the implementation of a type.
        /// </summary>
        /// <typeparam name="T">Type to retrieve</typeparam>
        /// <returns>Implementation of a type</returns>
        public T GetImplementationOf<T>() where T : class
        {
            // try to find it by type
            IDictionary registeredObjects = _applicationContext.GetObjectsOfType(typeof(T));
            if (registeredObjects.Count == 0)
            {
                // try to get if from XMLApplicationContext
                registeredObjects = _applicationContext.ParentContext.GetObjectsOfType(typeof(T));
            }

            // evaluate what we found
            if (registeredObjects.Count > 1)
            {
                throw new Spring.Objects.Factory.UnsatisfiedDependencyException(
                    string.Format(ContainerResources.MoreThanOneElementRegistered, typeof(T).Name));
            }

            if (registeredObjects.Count == 0)
            {
                throw new Spring.Objects.Factory.UnsatisfiedDependencyException(
                    string.Format(ContainerResources.NoElementsRegistered, typeof(T).Name));
            }

            // take first element
            return registeredObjects.Values.Cast<T>().FirstOrDefault();
        }

        #region IDisposable Members

        public void Dispose()
        {
            Dispose(true);
            // Take yourself off the Finalization queue 
            // to prevent finalization code for this object
            // from executing a second time.
            GC.SuppressFinalize(this);
        }

        // Dispose(bool disposing) executes in two distinct scenarios.
        // If disposing equals true, the method has been called directly
        // or indirectly by a user's code. Managed and unmanaged resources
        // can be disposed.
        // If disposing equals false, the method has been called by the 
        // runtime from inside the finalizer and you should not reference 
        // other objects. Only unmanaged resources can be disposed.
        protected virtual void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!_disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    if (_applicationContext != null)
                        _applicationContext.Dispose();
                }
                // Release unmanaged resources. If disposing is false, 
                // only the following code is executed.
                // Note that this is not thread safe.
                // Another thread could start disposing the object
                // after the managed resources are disposed,
                // but before the disposed flag is set to true.
                // If thread safety is necessary, it must be
                // implemented by the client.
            }
            _disposed = true;
        }

        // Use C# destructor syntax for finalization code.
        // This destructor will run only if the Dispose method 
        // does not get called.
        // It gives your base class the opportunity to finalize.
        // Do not provide destructors in types derived from this class.
        ~SpringContainer()
        {
            // Do not re-create Dispose clean-up code here.
            // Calling Dispose(false) is optimal in terms of
            // readability and maintainability.
            Dispose(false);
        }

        #endregion
    }
}
