﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Diagnostics;
using Canonical.UbuntuOne.Common.Validation;
using log4net;

namespace Canonical.UbuntuOne.Common
{
    internal class ProcessManager : IProcessManager
    {
        #region Variables

        private Process _process;
        private readonly object _loggerLock = new object();
        private ILog _logger;

        #endregion

        #region Properties

        /// <summary>
        /// Gets and sets the path of the command to execute.
        /// </summary>
        internal string CommandPath { get; set; }

        /// <summary>
        /// Gets and sets the parameters to be passed to the pararameter.
        /// </summary>
        internal string Arguments { get; set; }

        /// <summary>
        /// Gets and sets if the console should be shown when the process is executed.
        /// </summary>
        internal bool ShowConsole { get; set; }

        /// <summary>
        /// Gets and sets the authentication object that wukk ensure that the process is ran with the 
        /// correct credentials.
        /// </summary>
        internal IAuthentication Authentication { get; set; }

        /// <summary>
        /// Gets the logger to used with the object.
        /// </summary>
        internal ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(ProcessManager));
                    }
                }
                return _logger;
            }
            set
            {
                _logger = value;
            }
        }
        #endregion

        internal ProcessManager()
        {
            IsRunning = false;
        }

        /// <summary>
        /// Implementators of this method should start the process and throw an exception if it was not possible.
        /// </summary>
        /// <exception cref="ProcessManagementException">This exception is thrown when the process could not be started.</exception>
        public void Start()
        {
            ValidateArgs.Begin()
                .IsNotNull(CommandPath)
                .IsNotNull(Arguments)
                .Check();

            var info = new ProcessStartInfo
            {
                Arguments = Arguments,
                CreateNoWindow = true,
                FileName = CommandPath,
                UserName = Authentication.Username
            };
            if (_process == null)
            {
                _process = new Process { StartInfo = info };
                _process.Start();
            }
            else
            {
                var mesage = String.Format("Process {0} for user {1} was already started.", CommandPath,
                                           Authentication.Username);
                Logger.ErrorFormat(mesage);
                throw new ProcessManagementException(mesage);
            }
        }

        /// <summary>
        /// Implementators of this method should start a process and return a boolean value stating if
        /// the process was correctly started. Using this method allows to avoid the need to catch an dexception.
        /// </summary>
        /// <returns>Booelan flag stating if the process was indeed started.</returns>
        public bool TryStart()
        {
            try
            {
                Start();
                return true;
            }
            catch(ProcessManagementException)
            {
                return false;
            }
        }

        /// <summary>
        /// Implementators of this method should stop a process that has been currently running.
        /// </summary>
        /// <exception cref="ProcessManagementException">This exception is thrown when the process has not yet been started.</exception>
        public void Stop()
        {
            ValidateArgs.Begin()
                .IsNotNull(CommandPath)
                .IsNotNull(Arguments)
                .Check();
            if(_process != null)
            {
                _process.Close();
            }
            else
            {
                var message = String.Format("Process {0} is not running for user {1}.", CommandPath,
                                            Authentication.Username);
                Logger.ErrorFormat(message);
                throw new ProcessManagementException(message);
            }
        }

        /// <summary>
        /// Implementators of this method should ensure that the process can be stopped, if it cannot be stopped they should
        /// return a boolean flag rather than raising an exception.
        /// </summary>
        /// <returns></returns>
        public bool TryStop()
        {
            try
            {
                Stop();
                return true;
            }
            catch(ProcessManagementException)
            {
                return false;
            }
        }

        /// <summary>
        /// Returns if the process has already been started.
        /// </summary>
        public bool IsRunning { get; private set; }
    }
}
