﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace Canonical.UbuntuOne.Common.Validation
{

    /// <summary>
    /// Contains all the validations that are related with the comparison of the arguments with an other value.
    /// </summary>
    public static class EqualityValidationExtensions
    {
        /// <summary>
        /// Validation that ensures that the argument is greated that an other given value.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the validations done so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="T"/> to be tested.
        /// </param>
        /// <param name="otherValue">
        /// A <see cref="T"/> with the value to test agains.
        /// </param>
        /// <param name="parameterName">
        /// A <see cref="System.String"/> with the message to be used in the exception.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation IsGreaterThan<T>(this ArgumentValidation argumentValidation,
            T value, T otherValue, string parameterName) where T : IComparable, IComparable<T>
        {
            return value.CompareTo(otherValue) > 0 ?
            argumentValidation :
            (argumentValidation ?? new ArgumentValidation()).AddException(
                new ArgumentException(string.Format("'{0}' must be greater than {1}, but was {2}.",
                    parameterName, otherValue, value)));
        }

        /// <summary>
        /// Validation that ensures that the argument is greater or equal to a given value.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the validations done so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="T"/> with the argument to validate.
        /// </param>
        /// <param name="otherValue">
        /// A <see cref="T"/> with the value to compare against.
        /// </param>
        /// <param name="parameterName">
        /// A <see cref="System.String"/> with the message to be used in the exception.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation IsGreaterOrEqualThan<T>(this ArgumentValidation argumentValidation, T value,
            T otherValue, string parameterName) where T : IComparable, IComparable<T>
        {
            return value.CompareTo(otherValue) >= 0 ?
            argumentValidation :
            (argumentValidation ?? new ArgumentValidation()).AddException(
                new ArgumentException(string.Format("'{0}' must be greater or equal than {1}, but was {2}.",
                    parameterName, otherValue, value)));
        }

        /// <summary>
        /// Ensures that the arguments equals the given value.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the validations so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="T"/> to be validated.
        /// </param>
        /// <param name="otherValue">
        /// A <see cref="T"/> with the valut to validate against.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation IsEqual<T>(this ArgumentValidation argumentValidation, T value, T otherValue)
            where T : IComparable
        {
            return value.CompareTo(otherValue) == 0 ?
                argumentValidation :
                (argumentValidation ?? new ArgumentValidation()).AddException(new ArgumentException(
                    string.Format("'{0}' must be equal '{1}'.", value, otherValue)));
        }

        /// <summary>
        /// Ensures that the argument equals all the object in the given array.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the validation so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="T"/> with the argument to validate.
        /// </param>
        /// <param name="otherValues">
        /// A <see cref="ICollection"/> with the values to validate against.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation IsEqual<T>(this ArgumentValidation argumentValidation, T value,
            ICollection<T> otherValues) where T : IComparable
        {
            var result = false;
            foreach (var current in otherValues)
            {
                result |= value.CompareTo(current) == 0;
                if (!result)
                    break;
            }
            return result ?
                argumentValidation :
                (argumentValidation ?? new ArgumentValidation()).AddException(new ArgumentException(
                    string.Format("'{0}' must be equal to one of the values in the array.", value)));
        }

        /// <summary>
        /// Ensures that the argument does not equals the given value.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the validation so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="T"/> with the argument to validate.
        /// </param>
        /// <param name="otherValue">
        /// A <see cref="T"/> with the value to validate against.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation IsNotEqual<T>(this ArgumentValidation argumentValidation, T value, T otherValue)
            where T : IComparable
        {
            return value.CompareTo(otherValue) != 0 ?
                argumentValidation :
                (argumentValidation ?? new ArgumentValidation()).AddException(new ArgumentException(
                    string.Format("'{0}' not must bew equal '{1}'.", value, otherValue)));
        }

        /// <summary>
        /// Ensures that the argument is not equal to any of the values found in the collection.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the valdiation result so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="T"/> with the argument to validate.
        /// </param>
        /// <param name="otherValue">
        /// A <see cref="ICollection"/> with the value to validate against.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation IsNotEqual<T>(this ArgumentValidation argumentValidation, T value,
            ICollection<T> otherValues) where T : IComparable
        {
            var result = true;
            foreach (var current in otherValues)
            {
                result &= value.CompareTo(current) != 0;
                if (!result)
                    break;
            }
            return result ?
                argumentValidation :
                (argumentValidation ?? new ArgumentValidation()).AddException(new ArgumentException(
                    string.Format("'{0}' not must bew equal to a value in the array.", value)));
        }

        /// <summary>
        /// Ensures that the argument matches the given exception.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the current result of the validations.
        /// </param>
        /// <param name="value">
        /// A <see cref="System.String"/> with the argument to validate.
        /// </param>
        /// <param name="regularExpression">
        /// A <see cref="Regex"/> with the regular expression to match.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> the result of the validation.
        /// </returns>
        public static ArgumentValidation Matches(this ArgumentValidation argumentValidation, string value,
            Regex regularExpression)
        {
            return (regularExpression.IsMatch(value)) ?
                argumentValidation :
                (argumentValidation ?? new ArgumentValidation()).AddException(new ArgumentException(
                    string.Format("'{0}' must match '{1}'.", value, regularExpression)));
        }

        public static ArgumentValidation IsNotNullOrEmpty(this ArgumentValidation argumentValidation, string  value,
            string parameterName)
        {
            return (!String.IsNullOrEmpty(value))
                ? argumentValidation:
                (argumentValidation ?? new ArgumentValidation()).AddException(new ArgumentException(
                    string.Format("'{0}' cannot be null or empty.", parameterName)));
        }
    }
}
