﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Security.Permissions;
using System.Text;

namespace Canonical.UbuntuOne.Common.Validation
{
    /// <summary>
    ///Exception used to hold more than one exception to allow several erros in the validations. 
    /// </summary>
    [Serializable]
    public sealed class MultiException : Exception
    {
        private readonly Exception[] _innerExceptions;

        /// <value>
        /// A collections with the inner exceptions. 
        /// </value>
        public IEnumerable<Exception> InnerExceptions
        {
            get
            {
                if (_innerExceptions != null)
                {
                    foreach (var t in _innerExceptions)
                    {
                        yield return t;
                    }
                }
            }
        }

        /// <summary>
        ///Creates a new exception. 
        /// </summary>
        public MultiException()
        {
        }

        /// <summary>
        /// Creates a new exception with the given messsage. 
        /// </summary>
        /// <param name="message">
        /// A <see cref="System.String"/> with the message of the exception.
        /// </param>
        public MultiException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Creates a new exceptionwih the givenmessage and inner exception.
        /// </summary>
        /// <param name="message">
        /// A <see cref="System.String"/> with the message of the exception.
        /// </param>
        /// <param name="innerException">
        /// A <see cref="Exception"/> that represents the inner exception.
        /// </param>
        public MultiException(string message, Exception innerException)
            : base(message, innerException)
        {
            _innerExceptions = new[] { innerException };
        }

        /// <summary>
        /// Creates an exception with a collection of inner exceptions. 
        /// </summary>
        /// <param name="innerExceptions">
        /// A <see cref="IEnumerable" /> with the inner exceptions.
        /// </param>
        public MultiException(IEnumerable<Exception> innerExceptions)
            : this(null, innerExceptions)
        {
        }

        /// <summary>
        /// Creates an exception with as collection of inner exceptions.
        /// </summary>
        /// <param name="innerExceptions">
        /// A <see cref="Exception"/> with the inner exceptions.
        /// </param>
        public MultiException(Exception[] innerExceptions)
            : this(null, (IEnumerable<Exception>)innerExceptions)
        {
        }

        /// <summary>
        /// Creates a new exception with the given message and inner exceptions.
        /// </summary>
        /// <param name="message">
        /// A <see cref="System.String"/> with the message of the exception.
        /// </param>
        /// <param name="innerExceptions">
        /// A <see cref="Exception"/> with the collection of inner exceptions.
        /// </param> 
        public MultiException(string message, Exception[] innerExceptions)
            : this(message, (IEnumerable<Exception>)innerExceptions)
        {
        }

        /// <summary>
        /// Creates a new exception with the given message and exceptions.
        /// </summary>
        /// <param name="message">
        /// A <see cref="System.String"/> with the message of the exception.
        /// </param>
        /// <param name="innerExceptions">
        /// A <see cref="IEnumerable"/> with the inner exceptions.
        /// </param>
        public MultiException(string message, IEnumerable<Exception> innerExceptions)
            : base(message, innerExceptions.FirstOrDefault())
        {
            foreach (var exception in innerExceptions)
                if (exception == null)
                    throw new ArgumentNullException("innerExceptions");

            _innerExceptions = innerExceptions.ToArray();
        }

        /// <summary>
        /// Creates a new exception from the serialization info.
        /// </summary>
        /// <param name="info">
        /// A <see cref="SerializationInfo"/> with the info of the exception.
        /// </param>
        /// <param name="context">
        /// A <see cref="StreamingContext"/> with the context of the exception.
        /// </param>
        private MultiException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        /// <summary>
        /// Returns the string representation of the exception.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents the exception.
        /// </returns>
        public override string ToString()
        {
            var builder = new StringBuilder(_innerExceptions.Length);
            foreach (var exception in _innerExceptions)
                builder.AppendLine(exception.ToString());
            return builder.ToString();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="info">
        /// A <see cref="SerializationInfo"/>
        /// </param>
        /// <param name="context">
        /// A <see cref="StreamingContext"/>
        /// </param>
        [SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);

            info.AddValue("CheckedOut", _innerExceptions);
        }
    }
}