﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Linq;

namespace Canonical.UbuntuOne.Common.Validation
{
    public static class ValidationExtensions
    {
        /// <summary>
        /// Validation that ensures that the argument is not null. 
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> that will be used when validating the arguments of a method.
        /// </param>
        /// <param name="theObject">
        /// A <see cref="T"/> to test if it is null.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation os far.
        /// </returns>
        public static ArgumentValidation IsNotNull<T>(this ArgumentValidation argumentValidation, T theObject)
             where T : class
        {
            return IsNotNull(argumentValidation, theObject, "value");
        }

        /// <summary>
        /// Validation that ensures that the argument is not null. 
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> that will be used when validating the arguments of a method.
        /// </param>
        /// <param name="theObject">
        /// A <see cref="T"/> to test if it is null.
        /// </param>
        /// <param name="parameterName">
        /// A <see cref="System.String"/> with the message to be used in the exception.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/>
        /// </returns>
        public static ArgumentValidation IsNotNull<T>(this ArgumentValidation argumentValidation,
            T theObject, string parameterName) where T : class
        {
            return ReferenceEquals(theObject, null) ?
                (argumentValidation ?? new ArgumentValidation()).AddException(
                    new ArgumentNullException(parameterName)) :
                argumentValidation;
        }

        /// <summary>
        /// Validation that ensures that the argument is true. Usually the argument comes from the result of an
        /// expression.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the validation so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="System.Boolean"/> to be tested.
        /// </param>
        /// <param name="message">
        /// A <see cref="System.String"/> with the message of the exception.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation IsTrue(this ArgumentValidation argumentValidation,
            bool value, String message)
        {
            return !value ?
                (argumentValidation ?? new ArgumentValidation()).AddException(
                    new ArgumentNullException(message)) :
                argumentValidation;
        }

        /// <summary>
        /// Validation that ensures that the argument is positive.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the result of the valdiation so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="System.Int64"/> with the argument to validate.
        /// </param>
        /// <param name="parameterName">
        /// A <see cref="System.String"/> with the name of the parameter being valdiated.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the valdiation.
        /// </returns>
        public static ArgumentValidation IsPositive(this ArgumentValidation argumentValidation,
            long value, string parameterName)
        {
            return value < 0 ?
                (argumentValidation ?? new ArgumentValidation()).AddException(
                    new ArgumentOutOfRangeException(parameterName,
                    string.Format("must be positive, but was {0}.", value))) :
                argumentValidation;
        }

        /// <summary>
        /// That validates that the value is of the correct type. 
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the result of the validation so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="System.Object"/> with the value to validate.
        /// </param>
        /// <param name="type">
        /// A <see cref="Type"/> with the type we validate against.
        /// </param>
        /// <param name="parameterName">
        /// A <see cref="System.String"/> with the message used for the exception.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation IsOfType(this ArgumentValidation argumentValidation, object value,
            Type type, string parameterName)
        {
            return !type.IsInstanceOfType(value) ?
                (argumentValidation ?? new ArgumentValidation()).AddException(new ArgumentException(
                    string.Format(">must be of the type {0}, but was {1}.", type, value.GetType()), parameterName)) :
                argumentValidation;
        }

        /// <summary>
        /// Validates that the time is utc.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the result of the validation so far.
        /// </param>
        /// <param name="value">
        /// A <see cref="DateTime"/> with the value to validate.
        /// </param>
        /// <param name="parameterName">
        /// A <see cref="System.String"/> with the name of the parameter to validate.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation IsUtcDateTime(this ArgumentValidation argumentValidation, DateTime value,
            string parameterName)
        {
            return value.Kind == DateTimeKind.Utc ?
                argumentValidation :
                (argumentValidation ?? new ArgumentValidation()).AddException(new ArgumentException(
                    string.Format("{0} must be UTC flagged DateTime.", value, parameterName)));
        }

        /// <summary>
        /// Performs the check to validate the results.
        /// </summary>
        /// <param name="argumentValidation">
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </param>
        /// <returns>
        /// A <see cref="ArgumentValidation"/> with the result of the validation.
        /// </returns>
        public static ArgumentValidation Check(this ArgumentValidation argumentValidation)
        {
            if (argumentValidation == null)
                return argumentValidation;

            if (argumentValidation.Exceptions.Take(2).Count() == 1)
                throw new ValidationException("There is one or more failed validations.",
                    argumentValidation.Exceptions.First());

            throw new ValidationException("There is one or more failed validations.",
                new MultiException(argumentValidation.Exceptions));
        }

    }
}