﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System.Collections.Generic;
using Canonical.UbuntuOne.Common.Validation;
using NUnit.Framework;

namespace Canonical.UbuntuOne.ProcessDispatcher.Tests
{
    /// <summary>
    /// Fixture used to ensure that the convert class performs the correct operations.
    /// </summary>
    [TestFixture]
    public class ConvertFixture
    {

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToStateDataWrongMessageTest()
        {
            var message = new SyncDaemonMessage
            {
                Type = SyncDaemonMessage.MessageType.CONNECT
            };
            Convert.ToStateData(message);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToStateDataNullPointerTest()
        {
            var message = new SyncDaemonMessage
            {
                Type = SyncDaemonMessage.MessageType.STATUS,
                State = null
            };
            Convert.ToStateData(message);
        }

        [Test]
        [TestCase("firstDescription", true, true, true, "name")]
        [TestCase("firstDescription", true, true, false, "name")]
        [TestCase("firstDescription", true, false, true, "name")]
        [TestCase("firstDescription", true, false, false, "name")]
        public void ToStateDataTest(string description, bool isConnected, bool isError,
            bool isOnline, string name)
        {
            var message = new SyncDaemonMessage
            {
                Type = SyncDaemonMessage.MessageType.STATUS,
                State = new SyncDaemonMessage.StateMessage
                            {
                                Description = description,
                                IsConnected = isConnected,
                                IsError = isError,
                                IsOnline = isOnline,
                                Name = name
                            }
            };
            var state = Convert.ToStateData(message);
            Assert.AreEqual(description, state.Description);
            Assert.AreEqual(isConnected, state.IsConnected);
            Assert.AreEqual(isError, state.IsError);
            Assert.AreEqual(isOnline, state.IsOnline);
            Assert.AreEqual(name, state.Name);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToCurrentDownloadsWrongMessageTest()
        {
            var message = new SyncDaemonMessage
            {
                Type = SyncDaemonMessage.MessageType.SCHEDULENEXT
            };
            Convert.ToCurrentDownloads(message);
        }


        [TestCase("first", 3, "second", 5)]
        [TestCase("thrid", 65, "last", 45)]
        public void ToCurrentDownloadsTest(string firstData, int firstRead, string secondData, int secondRead)
        {
            var message = new SyncDaemonMessage(new List<SyncDaemonMessage.DownloadMessage>())
            {
                Type = SyncDaemonMessage.MessageType.DOWNLOADS,
            };
            message.Downloads.Add(new SyncDaemonMessage.DownloadMessage
            {
                NumberOfBytesRead = firstRead,
                Node = new SyncDaemonMessage.NodeMessage
                {
                    Path = firstData,
                    VolumeId = firstData,
                    NodeId = firstData
                }
            });
            message.Downloads.Add(new SyncDaemonMessage.DownloadMessage
            {
                NumberOfBytesRead = secondRead,
                Node = new SyncDaemonMessage.NodeMessage
                {
                    Path = secondData,
                    VolumeId = secondData,
                    NodeId = secondData
                }
            });
            var converted = Convert.ToCurrentDownloads(message);
            Assert.AreEqual(message.Downloads.Count, converted.Count);
            foreach (var downloadData in converted)
            {
                Assert.IsTrue(downloadData.NodeId.Equals(firstData)
                    || downloadData.NodeId.Equals(secondData));
                if (downloadData.NodeId.Equals(firstData))
                {
                    Assert.AreEqual(firstData, downloadData.Path);
                    Assert.AreEqual(firstData, downloadData.VolumeId);
                    Assert.AreEqual(firstRead, downloadData.NumberOfBytesRead);
                }
                else
                {
                    // if it is different from secondData we would have a fail assertion before
                    Assert.AreEqual(secondData, downloadData.Path);
                    Assert.AreEqual(secondData, downloadData.VolumeId);
                    Assert.AreEqual(secondRead, downloadData.NumberOfBytesRead);
                }
            }
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToCurrentWaitingWrongMessageTest()
        {
            var message = new SyncDaemonMessage
            {
                Type = SyncDaemonMessage.MessageType.CONNECT
            };
            Convert.ToCurrentWaiting(message);
        }

        [TestCase("firstData", "secondData")]
        [TestCase("lastData", "thridData")]
        public void ToCurrentWaitingTest(string firstData, string secondData)
        {
            var message = new SyncDaemonMessage(new List<SyncDaemonMessage.QueuItemMessage>())
            {
                Type = SyncDaemonMessage.MessageType.WAITING
            };
            message.WaitingItems.Add(new SyncDaemonMessage.QueuItemMessage
            {
                Node = new SyncDaemonMessage.NodeMessage
                {
                    NodeId = firstData,
                    Path = firstData,
                    VolumeId = firstData
                },
                Operation = firstData,
            });
            message.WaitingItems.Add(new SyncDaemonMessage.QueuItemMessage
            {
                Node = new SyncDaemonMessage.NodeMessage
                {
                    NodeId = secondData,
                    Path = secondData,
                    VolumeId = secondData
                },
                Operation = secondData,
            });
            var converted = Convert.ToCurrentWaiting(message);
            foreach (var queueItemData in converted)
            {
                Assert.IsTrue(queueItemData.NodeId.Equals(firstData)
                    || queueItemData.NodeId.Equals(secondData));
                if (queueItemData.NodeId.Equals(firstData))
                {
                    Assert.AreEqual(firstData, queueItemData.Path);
                    Assert.AreEqual(firstData, queueItemData.Path);
                    Assert.AreEqual(firstData, queueItemData.Operation);
                }
                else
                {
                    // do not worry, other possible values will have thrown a fail assertion
                    Assert.AreEqual(secondData, queueItemData.Path);
                    Assert.AreEqual(secondData, queueItemData.Path);
                    Assert.AreEqual(secondData, queueItemData.Operation);
                }
            }
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToCurrentUploadsWrongMessage()
        {
            var message = new SyncDaemonMessage
            {
                Type = SyncDaemonMessage.MessageType.QUERYROOT
            };
            Convert.ToCurrentUploads(message);
        }

        [TestCase("firstData", 54, "secondData", 56)]
        [TestCase("data", 4, "lastData", 34)]
        public void ToCurrentUploadsTest(string firstData, int firstWrite, string secondData, int secondWrite)
        {
            var message = new SyncDaemonMessage(new List<SyncDaemonMessage.UploadMessage>())
            {
                Type = SyncDaemonMessage.MessageType.UPLOADS
            };
            message.Uploads.Add(new SyncDaemonMessage.UploadMessage
            {
                Node = new SyncDaemonMessage.NodeMessage
                {
                    Path = firstData,
                    NodeId = firstData,
                    VolumeId = firstData
                },
                NumberOfBytesWritten = firstWrite
            });
            message.Uploads.Add(new SyncDaemonMessage.UploadMessage
            {
                Node = new SyncDaemonMessage.NodeMessage
                {
                    Path = secondData,
                    NodeId = secondData,
                    VolumeId = secondData
                },
                NumberOfBytesWritten = secondWrite
            });
            var converted = Convert.ToCurrentUploads(message);
            foreach (var uploadData in converted)
            {
                Assert.IsTrue(uploadData.NodeId.Equals(firstData)
                    || uploadData.NodeId.Equals(secondData));
                if (uploadData.NodeId.Equals(firstData))
                {
                    Assert.AreEqual(firstWrite, uploadData.NumberOfBytesWritten);
                    Assert.AreEqual(firstData, uploadData.Path);
                    Assert.AreEqual(firstData, uploadData.VolumeId);
                }
                else
                {
                    // do not worry about other values we would have had a fail assertion.
                    Assert.AreEqual(secondWrite, uploadData.NumberOfBytesWritten);
                    Assert.AreEqual(secondData, uploadData.Path);
                    Assert.AreEqual(secondData, uploadData.VolumeId);
                }
            }
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToRootDirectoryWrongMessageTest()
        {
            var message = new SyncDaemonMessage
            {
                Type = SyncDaemonMessage.MessageType.CONNECT
            };
            Convert.ToRootDirectory(message);
        }

        [TestCase("firstRoot")]
        [TestCase("secondRoot")]
        public void ToRootDirecotryTest(string root)
        {
            var message = new SyncDaemonMessage
            {
                Type = SyncDaemonMessage.MessageType.ROOT,
                Root = root
            };
            var converted = Convert.ToRootDirectory(message);
            Assert.AreEqual(root, converted);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToThrottlingDataWrongMessageTest()
        {
            var message = new ConfigurationMessage
            {
                Type = ConfigurationMessage.MessageType.ENABLEAUTOSUBSCRIBE
            };
            Convert.ToThrottlingData(message);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToThrottlingDataNullPointerTest()
        {
            var message = new ConfigurationMessage
            {
                Type = ConfigurationMessage.MessageType.THROTTLING,
                Throttling = null
            };
            Convert.ToThrottlingData(message);
        }

        [TestCase(34, 45)]
        [TestCase(45, 32)]
        public void ToThrottlingDataTest(int download, int upload)
        {
            var message = new ConfigurationMessage
            {
                Type = ConfigurationMessage.MessageType.THROTTLING,
                Throttling = new ConfigurationMessage.ThrottlingLimitsMessage
                {
                    Download = download,
                    Upload = upload
                }
            };
            var converted = Convert.ToThrottlingData(message);
            Assert.AreEqual(download, converted.Download);
            Assert.AreEqual(upload, converted.Upload);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToIsEnabledThrottlingWrongMessageTest()
        {
            var message = new ConfigurationMessage
            {
                Type = ConfigurationMessage.MessageType.QUERYAUTOSUBSCRIBEENABLED
            };
            Convert.ToIsEnabledThrottling(message);
        }

        [TestCase(true)]
        [TestCase(false)]
        public void ToIsEnabledThrottlingTest(bool isEnabled)
        {
            var message = new ConfigurationMessage
            {
                Type = ConfigurationMessage.MessageType.ENABLETHROTTLING,
                ThrottlingEnabled = isEnabled
            };
            var converted = Convert.ToIsEnabledThrottling(message);
            Assert.AreEqual(isEnabled, converted);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToIsAutosubscribeWrongMessage()
        {
            var message = new ConfigurationMessage
            {
                Type = ConfigurationMessage.MessageType.THROTTLING
            };
            Convert.ToIsAutosubscribe(message);
        }

        [TestCase(true)]
        [TestCase(false)]
        public void ToIsAutosubscribeTest(bool isEnabled)
        {
            var message = new ConfigurationMessage
            {
                Type = ConfigurationMessage.MessageType.ENABLEAUTOSUBSCRIBE,
                AutosubscribeEnabled = isEnabled
            };
            var converted = Convert.ToIsAutosubscribe(message);
            Assert.AreEqual(isEnabled, converted);
        }


        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToAllFoldesWrongMessage()
        {
            var message = new FolderMessage
            {
                Type = FolderMessage.MessageType.CREATEFOLDER
            };
            Convert.ToAllFolders(message);
        }

        [TestCase("firstData", true, "secondData", false)]
        [TestCase("oldData", false, "data", false)]
        public void ToAllFoldersTest(string firstData, bool firstSubscribed,
            string secondData, bool secondSubscribed)
        {
            var message = new FolderMessage(new List<FolderMessage.UdfDataMessage>())
            {
                Type = FolderMessage.MessageType.FOLDERS
            };
            message.Udfs.Add(new FolderMessage.UdfDataMessage
            {
                IsSubscribed = firstSubscribed,
                SuggestedPath = firstData,
                Node = new FolderMessage.NodeMessage
                {
                    NodeId = firstData,
                    VolumeId = firstData,
                    Path = firstData
                }
            });
            message.Udfs.Add(new FolderMessage.UdfDataMessage
            {
                IsSubscribed = secondSubscribed,
                SuggestedPath = secondData,
                Node = new FolderMessage.NodeMessage
                {
                    NodeId = secondData,
                    VolumeId = secondData,
                    Path = secondData
                }
            });
            var convert = Convert.ToAllFolders(message);
            Assert.AreEqual(message.Udfs.Count, convert.Count);
            foreach (var udfData in convert)
            {
                Assert.IsTrue(udfData.NodeId.Equals(firstData)
                    || udfData.NodeId.Equals(secondData));
                if (udfData.NodeId.Equals(firstData))
                {
                    Assert.AreEqual(firstSubscribed, udfData.IsSubscribed);
                    Assert.AreEqual(firstData, udfData.Path);
                    Assert.AreEqual(firstData, udfData.SuggestedPath);
                    Assert.AreEqual(firstData, udfData.VolumeId);
                }
                else
                {
                    Assert.AreEqual(secondSubscribed, udfData.IsSubscribed);
                    Assert.AreEqual(secondData, udfData.Path);
                    Assert.AreEqual(secondData, udfData.SuggestedPath);
                    Assert.AreEqual(secondData, udfData.VolumeId);
                }
            }
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void TestToUdfDataWrongMessage()
        {
            var message = new FolderMessage
            {
                Type = FolderMessage.MessageType.CREATEFOLDER
            };
            Convert.ToUdfData(message);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToUdfDataWrongNumberUdfs()
        {
            var message = new FolderMessage
            {
                Type = FolderMessage.MessageType.INFO
            };
            Convert.ToUdfData(message);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToUdfDataWrongNumber()
        {
            var message = new FolderMessage(new List<FolderMessage.UdfDataMessage>())
            {
                Type = FolderMessage.MessageType.INFO
            };
            Convert.ToUdfData(message);
        }

        [TestCase("firstData", true)]
        [TestCase("oldData", false)]
        public void ToUdfDataTest(string data, bool isSubscribed)
        {
            var message = new FolderMessage(new List<FolderMessage.UdfDataMessage>())
            {
                Type = FolderMessage.MessageType.INFO
            };
            message.Udfs.Add(new FolderMessage.UdfDataMessage
            {
                IsSubscribed = isSubscribed,
                SuggestedPath = data,
                Node = new FolderMessage.NodeMessage
                {
                    NodeId = data,
                    VolumeId = data,
                    Path = data
                }
            });
            var converted = Convert.ToUdfData(message);
            Assert.AreEqual(data, converted.NodeId);
            Assert.AreEqual(data, converted.Path);
            Assert.AreEqual(data, converted.SuggestedPath);
            Assert.AreEqual(data, converted.VolumeId);
            Assert.AreEqual(isSubscribed, converted.IsSubscribed);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToGetSharesWrongMessageTest()
        {
            var message = new ShareMessage
            {
                Type = ShareMessage.MessageType.REJECTSHARE
            };
            Convert.ToGetShares(message);
        }

        [TestCase("firstData", ShareMessage.AccessLevel.MODIFY, true, 43,
            "secondData", ShareMessage.AccessLevel.VIEW, false, 23)]
        [TestCase("oldData", ShareMessage.AccessLevel.VIEW, false, 3,
            "data", ShareMessage.AccessLevel.VIEW, false, 234)]
        public void ToGetSharesTest(string firstData, ShareMessage.AccessLevel firstAccess, bool firstAccepted,
            int firstFreeBytes, string secondData, ShareMessage.AccessLevel secondAccess, bool secondAccepted,
            int secondFreeBytes)
        {
            var message = new ShareMessage(new List<ShareMessage.ShareDataMessage>())
            {
                Type = ShareMessage.MessageType.OTHERSHARES
            };
            message.Shares.Add(new ShareMessage.ShareDataMessage
            {
                Access = firstAccess,
                FreeBytes = firstFreeBytes,
                IsAccepted = firstAccepted,
                Name = firstData,
                Node = new ShareMessage.NodeMessage
                {
                    Path = firstData,
                    NodeId = firstData,
                    VolumeId = firstData
                },
                OtherUsername = firstData,
                OtherVisibleName = firstData
            });
            message.Shares.Add(new ShareMessage.ShareDataMessage
            {
                Access = secondAccess,
                FreeBytes = secondFreeBytes,
                IsAccepted = secondAccepted,
                Name = secondData,
                Node = new ShareMessage.NodeMessage
                {
                    Path = secondData,
                    NodeId = secondData,
                    VolumeId = secondData
                },
                OtherUsername = secondData,
                OtherVisibleName = secondData
            });
            var converted = Convert.ToGetShares(message);
            Assert.AreEqual(message.Shares.Count, converted.Count);
            foreach (var shareData in converted)
            {
                Assert.IsTrue(shareData.NodeId.Equals(firstData)
                    || shareData.NodeId.Equals(secondData));
                if(shareData.NodeId.Equals(firstData))
                {
                    Assert.AreEqual(firstData, shareData.Name);
                    Assert.AreEqual(firstData, shareData.OtherUserName);
                    Assert.AreEqual(firstData, shareData.OtherVisisbleName);
                    Assert.AreEqual(firstData, shareData.Path);
                    Assert.AreEqual(firstData, shareData.VolumeId);
                    Assert.AreEqual(firstAccess.ToString(), shareData.AccessLevel.ToString());
                    Assert.AreEqual(firstFreeBytes, shareData.FreeBytes);
                    Assert.AreEqual(firstAccepted, shareData.IsAccepted);
                }
                else
                {
                    Assert.AreEqual(secondData, shareData.Name);
                    Assert.AreEqual(secondData, shareData.OtherUserName);
                    Assert.AreEqual(secondData, shareData.OtherVisisbleName);
                    Assert.AreEqual(secondData, shareData.Path);
                    Assert.AreEqual(secondData, shareData.VolumeId);
                    Assert.AreEqual(secondAccess.ToString(), shareData.AccessLevel.ToString());
                    Assert.AreEqual(secondFreeBytes, shareData.FreeBytes);
                    Assert.AreEqual(secondAccepted, shareData.IsAccepted);
                }
            }
        }


        #region ISyncDaemonClient conversions

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToDownloadDataWrongMessage()
        {
            var message = new SyncDaemonClientMessage
            {
                Type = SyncDaemonClientMessage.MessageType.NEWSHARE
            };
            Convert.ToDownloadData(message);
        }

        [TestCase("firstData", 43, SyncDaemonClientMessage.MessageType.DOWNLOADSTARTED)]
        [TestCase("secondData", 2, SyncDaemonClientMessage.MessageType.DOWNLOADFINISHED)]
        public void ToDownloadDataTest(string data, int numberOfBytes, SyncDaemonClientMessage.MessageType messageType)
        {
            var message = new SyncDaemonClientMessage
            {
                Type = SyncDaemonClientMessage.MessageType.DOWNLOADSTARTED,
                Download = new SyncDaemonClientMessage.DownloadMessage
                {
                    Node = new SyncDaemonClientMessage.NodeMessage
                    {
                        Path = data,
                        VolumeId = data,
                        NodeId = data
                    },
                    NumberOfBytesRead = numberOfBytes
                }
            };
            var converted = Convert.ToDownloadData(message);
            Assert.AreEqual(data, converted.NodeId);
            Assert.AreEqual(data, converted.Path);
            Assert.AreEqual(data, converted.VolumeId);
            Assert.AreEqual(numberOfBytes, converted.NumberOfBytesRead);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToUploadDataWrongMessageTest()
        {
            var message = new SyncDaemonClientMessage
            {
                Type = SyncDaemonClientMessage.MessageType.DOWNLOADSTARTED
            };
            Convert.ToUploadData(message);
        }

        [TestCase("firstData", 23, SyncDaemonClientMessage.MessageType.UPLOADSTARTED)]
        [TestCase("secondData", 255, SyncDaemonClientMessage.MessageType.UPLOADFINISHED)]
        public void ToUploadDataTest(string data, int bytesWritten, SyncDaemonClientMessage.MessageType messageType)
        {
            var message = new SyncDaemonClientMessage
            {
                Type = messageType,
                Upload = new SyncDaemonClientMessage.UploadMessage
                {
                    Node = new SyncDaemonClientMessage.NodeMessage
                    {
                        Path =  data,
                        NodeId = data,
                        VolumeId = data
                    },
                    NumberOfBytesWritten = bytesWritten
                }
            };
            var convert = Convert.ToUploadData(message);
            Assert.AreEqual(data, convert.NodeId);
            Assert.AreEqual(data, convert.Path);
            Assert.AreEqual(data, convert.VolumeId);
            Assert.AreEqual(bytesWritten, convert.NumberOfBytesWritten);
        }

        [Test]
        [ExpectedException(typeof(ValidationException))]
        public void ToShareDataWrongMessageType()
        {
            var message = new SyncDaemonClientMessage
            {
                Type = SyncDaemonClientMessage.MessageType.REJECTSHARESUCCESS
            };
            Convert.ToShareData(message);
        }

        [TestCase(SyncDaemonClientMessage.MessageType.NEWSHARE, SyncDaemonClientMessage.AccessLevel.MODIFY, 
            34, true, "firstData")]
        [TestCase(SyncDaemonClientMessage.MessageType.SHARECHANGED, SyncDaemonClientMessage.AccessLevel.MODIFY,
            345, false, "secondData")]
        public void ToShareDataTest(SyncDaemonClientMessage.MessageType messageType, 
            SyncDaemonClientMessage.AccessLevel access, int freeBytes, bool isAccepted, string data)
        {
            var message = new SyncDaemonClientMessage
            {
                Type = messageType,
                Share = new SyncDaemonClientMessage.ShareMessage
                {
                    AccessLevel = access,
                    FreeBytes = freeBytes,
                    IsAccepted = isAccepted,
                    Name = data,
                    Node = new SyncDaemonClientMessage.NodeMessage
                    {
                        NodeId = data,
                        VolumeId = data,
                        Path = data
                    },
                    OtherUsername = data,
                    OtherVisibleName = data
               }
            };
            var convert = Convert.ToShareData(message);
            Assert.AreEqual(freeBytes, convert.FreeBytes);
            Assert.AreEqual(isAccepted, convert.IsAccepted);
            Assert.AreEqual(data, convert.Name);
            Assert.AreEqual(data, convert.NodeId);
            Assert.AreEqual(data, convert.OtherUserName);
            Assert.AreEqual(data, convert.OtherVisisbleName);
            Assert.AreEqual(data, convert.Path);
            Assert.AreEqual(data, convert.VolumeId);
        }

        #endregion
    }
}
