﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using Canonical.UbuntuOne.Common;
using log4net;
using NUnit.Framework;
using Rhino.Mocks;

namespace Canonical.UbuntuOne.ProcessDispatcher.Tests
{
    [TestFixture]
    public class SyncDaemonServiceFixture
    {
        #region Setup

        private MockRepository _mocks;
        private IAuthentication _auth;
        private IProcessManagerFactory _managerFactory;
        private IProcessManager _processManager;
        private SyncDaemonService _service;
        private ILog _logger;

        [SetUp]
        public void Setup()
        {
            _mocks = new MockRepository();
            _auth = _mocks.DynamicMock<IAuthentication>();
            _managerFactory = _mocks.DynamicMock<IProcessManagerFactory>();
            _processManager = _mocks.DynamicMock<IProcessManager>();
            _logger = _mocks.DynamicMock<ILog>();
            _service = new SyncDaemonService
            {
                Logger = _logger,
                ProcessManagerFactory = _managerFactory,
                Authentication = _auth
            };
        }

        #endregion

        #region Tests

        /// <summary>
        /// This test ensures that the different componets used by the service are called 
        /// correctly in order when we are following the default path where everything went ok.
        /// </summary>
        [Test]
        public void StartTest()
        {
            using (_mocks.Record())
            {
                // expect to call the authentication class as well as
                // the factory and the manager.
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(true)
                    .Repeat.Once();

                SetupResult.For(_auth.Username)
                    .Return("mandel")
                    .Repeat.Once();

                Expect.Call(_managerFactory.GetManager())
                    .Return(_processManager)
                    .Repeat.Once();

                Expect.Call(() => _processManager.Start())
                    .Repeat.Once();
                    
            }
            using (_mocks.Playback())
            {
                _service.Start();
            }
        }

        /// <summary>
        /// This tests makes sure that the TryStart method works correctly when the 
        /// start method works correctly.
        /// </summary>
        [Test]
        public void TryStartSuccessTest()
        {
            using (_mocks.Record())
            {
                // expect to call the authentication class as well as
                // the factory and the manager.
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(true)
                    .Repeat.Once();

                SetupResult.For(_auth.Username)
                    .Return("mandel")
                    .Repeat.Once();

                Expect.Call(_managerFactory.GetManager())
                    .Return(_processManager)
                    .Repeat.Once();

                Expect.Call(() => _processManager.Start())
                    .Repeat.Once();

            }
            using (_mocks.Playback())
            {
                Assert.IsTrue(_service.TryStart());
            }
        }

        /// <summary>
        /// This test ensures that we cover the path in which we did not managed to start the process due
        /// to an issue with the process manager.
        /// </summary>
        [Test]
        [ExpectedException(typeof(SyncDaemonException))]
        public void StartExceptionTest()
        {
            using (_mocks.Record())
            {
                // we try to run the process but we will throw an exception when
                // trying to start the process
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(true)
                    .Repeat.Once();

                SetupResult.For(_auth.Username)
                    .Return("mandel")
                    .Repeat.Once();

                Expect.Call(_managerFactory.GetManager())
                    .Return(_processManager)
                    .Repeat.Once();

                Expect.Call(() => _processManager.Start())
                    .Throw(new ProcessManagementException());
            }

            using (_mocks.Playback())
            {
                _service.Start();
            }
        }

        /// <summary>
        /// This test makes sure that the path that is used when we could not start the test works
        /// as expected and returns a false.
        /// </summary>
        [Test]
        public void TryStartFailTest()
        {
            using (_mocks.Record())
            {
                // we try to run the process but we will throw an exception when
                // trying to start the process
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(true)
                    .Repeat.Once();

                SetupResult.For(_auth.Username)
                    .Return("mandel")
                    .Repeat.Once();

                Expect.Call(_managerFactory.GetManager())
                    .Return(_processManager)
                    .Repeat.Once();

                Expect.Call(() => _processManager.Start())
                    .Throw(new ProcessManagementException());
            }

            using (_mocks.Playback())
            {
                Assert.False(_service.TryStart());
            }
        }

        /// <summary>
        /// This method tests that we do not start the process when the user is not the correct one.
        /// </summary>
        [Test]
        public void StartNotAuthenticatedTest()
        {
            using (_mocks.Record())
            {
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(false)
                    .Repeat.Once();

                Expect.Call(_managerFactory.GetManager())
                    .Repeat.Never();

                Expect.Call(() => _processManager.Start())
                    .Repeat.Never();
            }
            using (_mocks.Playback())
            {
                _service.Start();
            }
        }

        /// <summary>
        /// This method tests the path that will go through the normal behaviour of the stop method.
        /// </summary>
        [Test]
        public void StopTest()
        {
            using(_mocks.Record())
            {
                // set basic set up to be able to start and stop
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(true)
                    .Repeat.Once();

                SetupResult.For(_auth.Username)
                    .Return("mandel")
                    .Repeat.Once();

                Expect.Call(_managerFactory.GetManager())
                    .Return(_processManager)
                    .Repeat.Once();

                Expect.Call(() => _processManager.Stop())
                    .Repeat.Once();
            }

            using (_mocks.Playback())
            {
                _service.Start();
                _service.Stop();
            }
        }

        /// <summary>
        /// This test ensures that the path used when we can stop does work as expected and returns a true.
        /// </summary>
        [Test]
        public void TryStopSuccessTest()
        {
            using (_mocks.Record())
            {
                // set basic set up to be able to start and stop
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(true)
                    .Repeat.Once();

                SetupResult.For(_auth.Username)
                    .Return("mandel")
                    .Repeat.Once();

                Expect.Call(_managerFactory.GetManager())
                    .Return(_processManager)
                    .Repeat.Once();

                Expect.Call(() => _processManager.Stop())
                    .Repeat.Once();
            }

            using (_mocks.Playback())
            {
                _service.Start();
                Assert.IsTrue(_service.TryStop());
            }
        }

        /// <summary>
        /// This method tests the path in which we are going to try and stop a process that has 
        /// not been started.
        /// </summary>
        [Test]
        public void StopNotStartedTest()
        {
            using (_mocks.Record())
            {
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(true)
                    .Repeat.Once();

                SetupResult.For(_auth.Username)
                    .Return("mandel")
                    .Repeat.Once();
            }

            using (_mocks.Playback())
            {
                _service.Stop();
            }
        }

        /// <summary>
        /// This method tests that the path used when there is an issue with the process work as expected.
        /// </summary>
        [Test]
        [ExpectedException(typeof(SyncDaemonException))]
        public void StopProcessExceptionTest()
        {
            using (_mocks.Record())
            {
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(true)
                    .Repeat.Once();

                SetupResult.For(_auth.Username)
                    .Return("mandel")
                    .Repeat.Once();

                Expect.Call(_managerFactory.GetManager())
                    .Return(_processManager)
                    .Repeat.Once();

                Expect.Call(() => _processManager.Stop())
                    .Throw(new ProcessManagementException());
            }

            using (_mocks.Playback())
            {
                _service.Start();
                _service.Stop();
            }
        }

        /// <summary>
        /// This method test tha tpath that is used when there is n issue with the process and we cannot stop it.
        /// </summary>
        [Test]
        public void TryStopFailTEst()
        {

            using (_mocks.Record())
            {
                SetupResult.For(_auth.IsAuthenticated)
                    .Return(true)
                    .Repeat.Once();

                SetupResult.For(_auth.Username)
                    .Return("mandel")
                    .Repeat.Once();

                Expect.Call(_managerFactory.GetManager())
                    .Return(_processManager)
                    .Repeat.Once();

                Expect.Call(() => _processManager.Stop())
                    .Throw(new ProcessManagementException());
            }

            using (_mocks.Playback())
            {
                _service.Start();
                Assert.IsFalse(_service.TryStop());
            }
        }

        #endregion

    }
}
