/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System.Collections.Generic;
using System.Linq;
using Canonical.UbuntuOne.Common;
using Canonical.UbuntuOne.Common.Validation;
using log4net;

namespace Canonical.UbuntuOne.ProcessDispatcher
{
    /// <summary>
    /// This class provides a collection of method that allows to onvert a SyncDaemon message that
    /// used protobuf to talk with the python process into a data contract that the .Net languages 
    /// can use.
    /// </summary>
    internal static class Convert
    {
		private static ILog _logger = LogManager.GetLogger(typeof(Convert));
		
        #region ISycnDaemon related conversions

        /// <summary>
        /// Allows to extract the required information from a sync daemon message and stored in 
        /// a data contract that can be consummed by the .Net languages.
        /// </summary>
        /// <param name="daemonMessage">The sync daemon message that was returned by the 
        /// python process.</param>
        /// <returns>A sate data that can be consummed by the .Net languages.</returns>
        public static StateData ToStateData(SyncDaemonMessage daemonMessage)
        {
            ValidateArgs.Begin()
                .IsEqual(daemonMessage.Type, SyncDaemonMessage.MessageType.STATUS)
                .IsNotNull(daemonMessage.State, "daemonMessage.state")
                .Check();
            
            return new StateData
            {
                Description = daemonMessage.State.Description,
                IsConnected = daemonMessage.State.IsConnected,
                IsError = daemonMessage.State.IsError,
                IsOnline = daemonMessage.State.IsOnline,
                Name = daemonMessage.State.Name
            };
        }

        /// <summary>
        /// Allows to perform the conversion between a download proto message and a dowload data
        /// contract that can be consummed by .Net languages.
        /// </summary>
        /// <param name="downloadProto">The proto to be converted to a data contract.</param>
        /// <returns>A data contract that has the smae data as the one provided by the proto.</returns>
        private static DownloadData ToDownloadData(SyncDaemonMessage.DownloadMessage downloadProto)
        {
            ValidateArgs.Begin()
                .IsNotNull(downloadProto)
                .Check();

            return new DownloadData
            {
                NodeId = downloadProto.Node.NodeId,
                NumberOfBytesRead = downloadProto.NumberOfBytesRead,
                Path = downloadProto.Node.Path,
                VolumeId = downloadProto.Node.VolumeId
            };
        }

        /// <summary>
        /// Allows to convert a message that contains a collection of downloads to a 
        /// collection of data contracts that can be consummed by the .Net languages.
        /// </summary>
        /// <param name="daemonMessage">The message that contains the response of the 
        /// query to retrieve all the current downloads.</param>
        /// <returns>A collection of data contracts with the information that the message has.</returns>
        public static IList<DownloadData> ToCurrentDownloads (SyncDaemonMessage daemonMessage)
        {
                    	
        	ValidateArgs.Begin ()
                .IsEqual (daemonMessage.Type, SyncDaemonMessage.MessageType.DOWNLOADS)
                .IsNotNull (daemonMessage.Downloads, "daemonMessage.download")
                .Check ();

            return daemonMessage.Downloads.Select(proto => ToDownloadData(proto)).ToList();
        }

        /// <summary>
        /// Allows to perform the conversion between a queue item proto and a data contract that
        /// can be consummed by the .Net languages.
        /// </summary>
        /// <param name="queuItemProto">The proto whose information we are going to retrienve.</param>
        /// <returns>A data contract that can be consummed by the .Net languages.</returns>
        private static QueueItemData ToQueueItemData(SyncDaemonMessage.QueuItemMessage queuItemProto)
        {
            ValidateArgs.Begin()
                .IsNotNull(queuItemProto)
                .Check();

            return new QueueItemData
            {
                NodeId = queuItemProto.Node.NodeId,
                Operation = queuItemProto.Operation,
                Path = queuItemProto.Node.Path,
                VolumeId = queuItemProto.Node.VolumeId
            };
        }

        /// <summary>
        /// Allows to convert a sync daemon message to a collection of queue items. This allows
        /// to convert the proto message to a list of contracts that can be consummed by the .Net
        /// languages.
        /// </summary>
        /// <param name="daemonMessage">The message returned by the daemon.</param>
        /// <returns>A list of data contract that can be consummed by .Net languages.</returns>
        public static IList<QueueItemData> ToCurrentWaiting(SyncDaemonMessage daemonMessage)
        {
            ValidateArgs.Begin()
                .IsEqual(daemonMessage.Type, SyncDaemonMessage.MessageType.WAITING)
                .IsNotNull(daemonMessage.WaitingItems, "daemonMessage.waiting")
                .Check();

            return daemonMessage.WaitingItems.Select(proto => ToQueueItemData(proto)).ToList();
        }

        /// <summary>
        /// Allows to convert an upload proto in a data contract that can be used by
        /// the .Net languages.
        /// </summary>
        /// <param name="uploadProto">The upload proto that we are going to convert to a 
        /// data contract.</param>
        /// <returns>A data contrac that represents the data of the proto and that can
        /// be consummed by the .Net languages.</returns>
        private static UploadData ToUploadData(SyncDaemonMessage.UploadMessage uploadProto)
        {
            ValidateArgs.Begin()
                .IsNotNull(uploadProto)
                .Check();
            return new UploadData
            {
                NodeId = uploadProto.Node.NodeId,
                NumberOfBytesWritten = uploadProto.NumberOfBytesWritten,
                Path = uploadProto.Node.Path,
                VolumeId = uploadProto.Node.VolumeId
            };
        }

        /// <summary>
        /// Allows to convert an daemon message into a collection of data contrac that can 
        /// be consummed by the .Net languages.
        /// </summary>
        /// <param name="daemonMessage">The message comming from the daemon that we have
        /// to convert.</param>
        /// <returns>A collection of data contract that can be consummed by the .Net
        /// languages.</returns>
        public static IList<UploadData> ToCurrentUploads(SyncDaemonMessage daemonMessage)
        {
            ValidateArgs.Begin()
                .IsEqual(daemonMessage.Type, SyncDaemonMessage.MessageType.UPLOADS)
                .IsNotNull(daemonMessage.Uploads, "daemonMessage.upload")
                .Check();
            return daemonMessage.Uploads.Select(proto => ToUploadData(proto)).ToList();
        }

        #endregion

        #region ISyncConfiguration related conversions 

        /// <summary>
        /// Allows to convert a daemon message into a string with the root directory 
        /// used by the daemon.
        /// </summary>
        /// <param name="daemonMessage">The message returned by the daemon that contains 
        /// the root directory used by the daemon.</param>
        /// <returns>The root direcotry used by the daemon.</returns>
        public static string ToRootDirectory(SyncDaemonMessage daemonMessage)
        {
            ValidateArgs.Begin()
                .IsEqual(daemonMessage.Type, SyncDaemonMessage.MessageType.ROOT)
                .IsNotNull(daemonMessage.Root, "daemon.root")
                .Check();
            return daemonMessage.Root;
        }

        /// <summary>
        /// Allows to convert a configuration message into a Throlling data object that can 
        /// be consummed by the .Net languages. 
        /// </summary>
        /// <param name="configurationMessage">The message whose data is going to be converted.</param>
        /// <returns>A data contract that can be consummed by the .Net languages.</returns>
        public static ThrottlingData ToThrottlingData(ConfigurationMessage configurationMessage)
        {
            ValidateArgs.Begin()
                .IsEqual(configurationMessage.Type, ConfigurationMessage.MessageType.THROTTLING)
                .IsNotNull(configurationMessage.Throttling)
                .Check();
            return new ThrottlingData
            {
                Download = configurationMessage.Throttling.Download,
                Upload = configurationMessage.Throttling.Upload
            };
        }

        /// <summary>
        /// Allows to convert a configuration message into a boolean that indicates if the throttling has been 
        /// ebabled in the application or not.
        /// </summary>
        /// <param name="configurationMessage">The message whose data we are going to convert.</param>
        /// <returns>A boolean indicating if the throttling has been enabled in the daemon.</returns>
        public static bool ToIsEnabledThrottling(ConfigurationMessage configurationMessage)
        {
            ValidateArgs.Begin()
                .IsEqual(configurationMessage.Type, ConfigurationMessage.MessageType.ENABLETHROTTLING)
                .Check();
            return configurationMessage.ThrottlingEnabled;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="configurationMessage"></param>
        /// <returns></returns>
        public static bool ToIsAutosubscribe(ConfigurationMessage configurationMessage)
        {
            ValidateArgs.Begin()
                .IsEqual(configurationMessage.Type, ConfigurationMessage.MessageType.ENABLEAUTOSUBSCRIBE)
                .Check();
            return configurationMessage.AutosubscribeEnabled;
        }

        #endregion

        #region ISyncFolder realted conversions 

        /// <summary>
        /// 
        /// </summary>
        /// <param name="udfProto"></param>
        /// <returns></returns>
        private static UdfData ToUdfData(FolderMessage.UdfDataMessage udfProto)
        {
            ValidateArgs.Begin()
                .IsNotNull(udfProto)
                .Check();
            return new UdfData
            {
                IsSubscribed = udfProto.IsSubscribed,
                NodeId = udfProto.Node.NodeId,
                Path = udfProto.Node.Path,
                SuggestedPath = udfProto.SuggestedPath,
                VolumeId = udfProto.Node.VolumeId
            };
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="folderMessage"></param>
        /// <returns></returns>
        public static IList<UdfData> ToAllFolders(FolderMessage folderMessage)
        {
            ValidateArgs.Begin()
                .IsEqual(folderMessage.Type, FolderMessage.MessageType.FOLDERS)
                .IsNotNull(folderMessage.Udfs)
                .Check();
            return folderMessage.Udfs.Select(proto => ToUdfData(proto)).ToList();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="folderMessage"></param>
        /// <returns></returns>
        public static UdfData ToUdfData(FolderMessage folderMessage)
        {
            ValidateArgs.Begin()
                .IsEqual(folderMessage.Type, FolderMessage.MessageType.INFO)
                .IsNotNull(folderMessage.Udfs)
                .IsGreaterOrEqualThan(folderMessage.Udfs.Count, 1, "udfs count")
                .Check();
            var info = folderMessage.Udfs[0];
            return ToUdfData(info);
        }

        #endregion

        #region ISyncFileManager conversions

        // TODO: expose the file manager interface 

        #endregion

        #region ISyncShares conversions

        /// <summary>
        /// 
        /// </summary>
        /// <param name="message">
        /// </param>
        /// <returns>
        /// </returns>
        private static ShareData ToShareData(ShareMessage.ShareDataMessage message)
        {
			
            var data = new ShareData
			{
				FreeBytes = message.FreeBytes,
				IsAccepted = message.IsAccepted,
				Name = message.Name,
				NodeId = message.Node.NodeId,
				OtherUserName = message.OtherUsername,
				OtherVisisbleName = message.OtherVisibleName,
				Path = message.Node.Path,
				VolumeId = message.Node.Path
			};
			switch(message.Access)
			{
				case ShareMessage.AccessLevel.MODIFY:
					data.AccessLevel = AccessLevel.MODIFY;
					break;
				default:
					data.AccessLevel = AccessLevel.VIEW;
					break;
			}
			return data;
        }

		/// <summary>
		/// 
		/// </summary>
		/// <param name="message">
		/// </param>
		/// <returns>
		/// </returns>
		public static IList<ShareData> ToGetShares(ShareMessage message)
		{
			ValidateArgs.Begin()
				.IsEqual(ShareMessage.MessageType.OTHERSHARES, message.Type)
				.IsNotNull(message.Shares)
				.Check();
		    return message.Shares.Select(proto => ToShareData(proto)).ToList();
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="message">
		/// </param>
		/// <returns>
		/// </returns>
		public static IList<ShareData> ToGetShared(ShareMessage message)
		{
			ValidateArgs.Begin()
				.IsEqual(ShareMessage.MessageType.SHARED, message.Type)
				.IsNotNull(message.Shares)
				.Check();
		    return message.Shares.Select(proto => ToShareData(proto)).ToList();
		}
		
        #endregion
		
		#region ISyncDaemonClient conversions
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="message">
		/// 
		/// </param>
		/// <returns>
		/// 
		/// </returns>
		public static DownloadData ToDownloadData(SyncDaemonClientMessage message)
		{
			ValidateArgs.Begin()
				.IsTrue(SyncDaemonClientMessage.MessageType.DOWNLOADSTARTED.Equals(message.Type)
				        || SyncDaemonClientMessage.MessageType.DOWNLOADFINISHED.Equals(message.Type), 
                        "The message type is not correct.")
				//.IsNotNull(message.Download)
				.Check();
			return new DownloadData
			{
				NodeId = message.Download.Node.NodeId,
				Path = message.Download.Node.Path,
				NumberOfBytesRead = message.Download.NumberOfBytesRead,
				VolumeId = message.Download.Node.VolumeId
			};
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="message">
		/// 
		/// </param>
		/// <returns>
		/// 
		/// </returns>
		public static UploadData ToUploadData(SyncDaemonClientMessage message)
		{
			ValidateArgs.Begin()
				.IsTrue(SyncDaemonClientMessage.MessageType.UPLOADSTARTED == message.Type
				 		|| SyncDaemonClientMessage.MessageType.UPLOADFINISHED == message.Type, 
                        "The message type is not correct.")
				.IsNotNull(message.Upload)
				.Check();
			
			return new UploadData
			{
				NodeId = message.Upload.Node.NodeId,
				NumberOfBytesWritten = message.Upload.NumberOfBytesWritten,
				Path = message.Upload.Node.Path,
				VolumeId = message.Upload.Node.VolumeId
			};
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="message">
		/// 
		/// </param>
		/// <returns>
		/// 
		/// </returns>
		public static ShareData ToShareData(SyncDaemonClientMessage message)
		{
			ValidateArgs.Begin()
				.IsTrue(SyncDaemonClientMessage.MessageType.SHARECHANGED == message.Type
				        || SyncDaemonClientMessage.MessageType.NEWSHARE == message.Type, "The message type is not cor")
				.Check();
			var data = new ShareData
			{
				FreeBytes = message.Share.FreeBytes,
				IsAccepted = message.Share.IsAccepted,
				Name = message.Share.Name,
				NodeId = message.Share.Node.NodeId,
				OtherUserName = message.Share.OtherUsername,
				OtherVisisbleName = message.Share.OtherVisibleName,
				Path = message.Share.Node.Path,
				VolumeId = message.Share.Node.VolumeId
			};
			switch(message.Share.AccessLevel)
			{
				case SyncDaemonClientMessage.AccessLevel.MODIFY:
					data.AccessLevel = AccessLevel.MODIFY;
					break;
				default:
					data.AccessLevel = AccessLevel.VIEW;
					break;
			}
			return data;
		}
		
		#endregion

    }
}
