﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Collections.Generic;
using Canonical.UbuntuOne.Common;
using Canonical.UbuntuOne.Common.Validation;
using log4net;

namespace Canonical.UbuntuOne.ProcessDispatcher
{
    /// <summary>
    /// Implementation of the sync daemon interface that allows to manage the service from the windows service.
    /// </summary>
    internal class SyncDaemonService : ISyncDaemon
    {
        #region Variables

        // contains a map between user id and pid so that we know if the process is running already.
        private readonly IDictionary<string, IProcessManager> _processesMap;
        private readonly object _loggerLock = new object();
        private ILog _logger;

        #endregion

        #region Properties

        /// <summary>
        /// Allows to get and set the logger that will be used.
        /// </summary>
        internal ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(SyncDaemonService));
                    }
                }
                return _logger;
            }
            set
            {
                _logger = value;
            }
        }

        /// <summary>
        /// Allows to get and set the manager that will manage the sync daemon process.
        /// </summary>
        internal IProcessManagerFactory ProcessManagerFactory { get; set; }

        /// <summary>
        /// Allows to get and set the authentication object that allows to ensure that the user making the
        /// call is allowed.
        /// </summary>
        internal IAuthentication Authentication { get; set; }

        #endregion

        public SyncDaemonService()
        {
            _processesMap = new Dictionary<string, IProcessManager>();
        }

        /// <summary>
        /// Allows to start the daemon by a client.
        /// </summary>
        /// <exception cref="SyncDaemonException">Exception thrown when the process could not be started.</exception>
        public void Start()
        {
            ValidateArgs.Begin()
                .IsNotNull(Authentication)
                .Check();

            if (Authentication.IsAuthenticated)
            {
                var user = Authentication.Username;
                if (!_processesMap.ContainsKey(user))
                {
                    Logger.DebugFormat("Starting daemon for {0}.", user);
                    var manager = ProcessManagerFactory.GetManager();
                    try
                    {
                        // start the process and relate it to the user.
                        manager.Start();
                        _processesMap.Add(user, manager);
                    }
                    catch (ProcessManagementException ex)
                    {
                        var message = String.Format("Daemon for {0} could not be started: {1}", user, ex.Message);
                        Logger.ErrorFormat(message);
                        throw new SyncDaemonException(message, ex);
                    }
                }
                else
                {
                    Logger.WarnFormat("User {0} sync daemon is already running.", user);
                }
            }
        }

        /// <summary>
        /// Allows to try and start a daemon from the client. If the daemon cannot be started a false will be returned.
        /// </summary>
        /// <returns>A boolean flag telling if the daemon could be started.</returns>
        public bool TryStart()
        {
            // depend on the start method
            try
            {
                Start();
                return true;
            }
            catch(SyncDaemonException)
            {
                return false;
            }
        }

        /// <summary>
        /// Allows to stop the daemon by a client.
        /// </summary>
        /// <exception cref="SyncDaemonException">Exception thrown when the process could not be stopped.</exception>
        public void Stop()
        {
            ValidateArgs.Begin()
                .IsNotNull(Authentication)
                .Check();

            if (Authentication.IsAuthenticated)
            {
                var user = Authentication.Username;
                if(_processesMap.ContainsKey(user))
                {
                    // get the manager, stop the process and remove the info
                    var manager = _processesMap[user];
                    try
                    {
                        manager.Stop();
                        _processesMap.Remove(user);
                    }
                    catch(ProcessManagementException ex)
                    {
                        var message = String.Format("Daemon for {0} could not be stoped: {1}", user, ex.Message);
                        throw new SyncDaemonException(message, ex);
                    }
                }
                else
                {
                    Logger.WarnFormat("User {0} sync daemon is not running.", user);
                }
            }
        }

        /// <summary>
        /// Allows to try and stop the daemon from the client. If the daemon cannot be stopped a false will be returned.
        /// </summary>
        /// <returns>A boolean flag telling if the daemong could bs stopped.</returns>
        public bool TryStop()
        {
            try
            {
                Stop();
                return true;
            }
            catch(SyncDaemonException)
            {
                return false;
            }
        }

        /// <summary>
        /// Allows to query the service about the current status of the daemon.
        /// </summary>
        /// <returns>A data contract that contains the current status of the daemon.</returns>
        public StateData GetCurrentStatus()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows to query the service to return the downloads that are currently being performed.
        /// </summary>
        /// <returns>A list with all the current downloads being processed by the daemon.</returns>
        public IList<DownloadData> GetCurrentDownloads()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows to query the service to return the items that are either waiting from upload or download.
        /// </summary>
        /// <returns>The list of items that are in the queue to be processed.</returns>
        public IList<QueueItemData> GetWaitingContent()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows the client to schedule the share and node be next in the
        // queue of waiting commands.
        /// </summary>
        /// <param name="shareId">The unique id that describes the share.</param>
        /// <param name="nodeId">The unique id that describes the node.</param>
        public void ScheduleAsNext(string shareId, string nodeId)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows to query the aemon to retrieve all the different nodes that are currently neing uploaded.
        /// </summary>
        /// <returns>The list of the objects currently being uploaded.</returns>
        public IList<UploadData> GetCurrentUploads()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows the client to tell the daemon to connect. The daemon should not connect automatically, this
        /// should be explicitly stated by the client.
        /// </summary>
        public void Connect()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows the client to tell the daemon to disconnect.
        /// </summary>
        public void Disconnect()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows the client to query the root directoy used by the daemon.
        /// </summary>
        /// <returns></returns>
        public string GetRootDirectory()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows client to tell the sync daemon to quit. Discomment allows the daemon toquee the event
        /// for the next time it connects, quit completelly closes the service and stops listening the IO changes.
        /// </summary>
        public void Quit()
        {
            throw new NotImplementedException();
        }
    }
}
